package com.docus.soap.api;

import com.docus.soap.api.service.XMZSYYQRCodeAPIService;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.oxm.jaxb.Jaxb2Marshaller;
import org.springframework.ws.client.core.WebServiceTemplate;
import org.springframework.ws.client.support.interceptor.ClientInterceptor;
import org.springframework.ws.soap.SoapVersion;
import org.springframework.ws.soap.saaj.SaajSoapMessageFactory;
import org.springframework.ws.transport.http.HttpComponentsMessageSender;

import javax.inject.Named;
import java.util.ArrayList;
import java.util.List;

@Configuration
public class SoapAPIConfig {

    @Bean
    public WebServiceTemplate webServiceTemplate() {
        WebServiceTemplate webServiceTemplate = new WebServiceTemplate();
        webServiceTemplate.setMessageFactory(saajSoapMessageFactory());
        webServiceTemplate.setMarshaller(marshaller());
        webServiceTemplate.setUnmarshaller(unmarshaller());
        webServiceTemplate.setMessageSender(new HttpComponentsMessageSender());
        webServiceTemplate.setInterceptors(interceptors());
        return webServiceTemplate;
    }

    private ClientInterceptor[] interceptors() {
        List<ClientInterceptor> clientInterceptors = new ArrayList<>();
        clientInterceptors.add(new SoapAPILogIntercepter());
        return clientInterceptors.toArray(new ClientInterceptor[clientInterceptors.size()]);
    }

    @Bean
    public SaajSoapMessageFactory saajSoapMessageFactory() {
        SaajSoapMessageFactory factory = new SaajSoapMessageFactory();
        factory.setSoapVersion(SoapVersion.SOAP_11);
        return factory;
    }

    @Bean
    @Named(value = "marshaller")
    public Jaxb2Marshaller marshaller() {
        Jaxb2Marshaller marshaller = new Jaxb2Marshaller();
        marshaller.setBeanClassLoader(ClassLoader.getSystemClassLoader());
        marshaller.setPackagesToScan(new String[]{"com.docus.soap.api.request"});
        return marshaller;
    }

    @Bean
    @Named(value = "unmarshaller")
    public Jaxb2Marshaller unmarshaller() {
        Jaxb2Marshaller unmarshaller = new Jaxb2Marshaller();
        unmarshaller.setBeanClassLoader(ClassLoader.getSystemClassLoader());
        unmarshaller.setPackagesToScan(new String[]{"com.docus.soap.api.response"});
        return unmarshaller;
    }

    @Bean
    public SoapAPIMessageGenerator soapAPIMessageGenerator() {
        SoapAPIMessageGenerator soapApiMessageGenerator = new SoapAPIMessageGenerator();
        soapApiMessageGenerator.setMarshaller(marshaller());
        return soapApiMessageGenerator;
    }

    @Bean
    public SoapAPIResponseExtractor soapAPIResponseExtractor() {
        SoapAPIResponseExtractor soapApiResponseExtractor = new SoapAPIResponseExtractor();
        soapApiResponseExtractor.setUnmarshaller(unmarshaller());
        return soapApiResponseExtractor;
    }

    @Bean
    public SoapAPIClient soapApiClient() {
        SoapAPIClient soapAPIClient = new SoapAPIClient();
        soapAPIClient.setWebServiceTemplate(webServiceTemplate());
        soapAPIClient.setMessageGenerator(soapAPIMessageGenerator());
        soapAPIClient.setResponseExtractor(soapAPIResponseExtractor());
        return soapAPIClient;
    }

    @Bean
    public XMZSYYQRCodeAPIService complaintApiService() {
        XMZSYYQRCodeAPIService complaintApiService = new XMZSYYQRCodeAPIService();
        complaintApiService.setWebServiceClient(soapApiClient());
        return complaintApiService;
    }

}
